package tpv.tmc.lc.java.formplugin.form;

import com.itopview.wm.utils.YEM;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.form.control.events.BeforeItemClickEvent;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.math.BigDecimal;
import java.util.Date;
import java.util.EventObject;
import java.util.Optional;

/**
 * @Description: 当信用证金额改小，提交单据时需校验《发货单》{收款计划}【信用证号】对应【金额】='
 * 当前《收证处理》【金额】合计值必须大于等于改证【金额】*（1+【溢短装金额浮动比例（%）上限】）/100；
 * @Project: zhck
 * @Package: com.itopview.wm.sc.formplugin.form.OverAmountCheckedFormPlugin
 * @Date: 2023/11/18 18:29
 * @Author: Heisenberg Wenbei He
 * @Version: 1.0
 */
public class OverAmountCheckedFormPlugin extends AbstractBillPlugIn {
    private static final String AMOUNT = "amount"; // 收证金额
    private static final String UP_LIMIT = "amountscaleupper"; // 溢短装金额浮动比例上限
    private static final String SALE_SHIPMENTS = "tpv_dt_salesshipping"; // 销售发货
    private static boolean reasonable = true; // 是否合理
    private static final String USD = "tpv_usdrate"; // 美元原币
    private static final String RMB = "tpv_rmbrate"; // 人民币原币
    private static final String CURRENCY = "currency"; // 收证币别
    private static final String RMB_AMOUNT = "tpv_rmbamount"; // 开证申请折RMB金额
    private static final String USD_AMOUNT = "tpv_usdamount"; // 开证申请折USD金额
    private static final String RATE_TABLE = "tpv_exchangetable"; // 汇率表
    private static final String CURRENT_DATE = "bizdate"; // 汇率日期
    private static final String TO_RMBRATE = "tpv_exchangerate"; // 开证申请折RMB汇率
    private static final String TO_USDRATE = "tpv_tousdrate"; // 开证申请折USD汇率


    @Override
    public void afterBindData(EventObject e) {
        super.afterBindData(e);
        this.calculatRate();
    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        switch (e.getProperty().getName()) {
            case AMOUNT:
                overAmountCheck(e);
                break;
            default:
                break;
        }
    }

    @Override
    public void beforeItemClick(BeforeItemClickEvent evt) {
        super.beforeItemClick(evt);
        switch (evt.getItemKey()) {
            case "bar_save":
            case "bar_submit":
                if (!reasonable) {
                    evt.setCancel(true);
                    this.getView().showTipNotification("信用证金额必须大于等于改证【金额】*（1+【溢短装金额浮动比例（%）上限】）/100");
                }
                break;
            default:
                break;
        }
    }

    /**
     * @param e:
     * @author: Heisenberg
     * @date: 2023/11/18 19:08
     * @description: 比较《发货单》金额 与 《改证》金额
     */
    private void overAmountCheck(PropertyChangedArgs e) {
        BigDecimal oldValue = (BigDecimal) e.getChangeSet()[0].getOldValue();
        BigDecimal newValue = (BigDecimal) e.getChangeSet()[0].getNewValue();
        if (newValue.compareTo(oldValue) > 0) {
            BigDecimal upLimit = (BigDecimal) this.getModel().getValue(UP_LIMIT);
            BigDecimal end = newValue.multiply(upLimit.add(BigDecimal.ONE).divide(new BigDecimal(100)));
            DynamicObject saleShipment = BusinessDataServiceHelper.loadSingle(SALE_SHIPMENTS, new QFilter[]{
                    new QFilter("tpv_paymentmethods.tpv_lc.billno", QCP.equals, this.getModel().getValue("billno"))
            });
            String billno = (String) this.getModel().getValue("billno");
            isReasonable(saleShipment, billno, end);
        }
    }

    /**
     * @param saleShipment: 发货单单据
     * @param billno:       改证 单据编号
     * @param end:          改证后计算后的金额
     * @author: Heisenberg
     * @date: 2023/11/24 18:01
     * @description: 当信用证金额改小，提交单据时需校验《发货单》{收款计划}【信用证号】对应【金额】='
     * * 当前《收证处理》【金额】合计值必须大于等于改证【金额】*（1+【溢短装金额浮动比例（%）上限】）/100；
     */
    private static void isReasonable(DynamicObject saleShipment, String billno, BigDecimal end) {
        Optional<BigDecimal> totalAmount = Optional.ofNullable(saleShipment)
                .map(payway -> payway.getDynamicObjectCollection("tpv_paymentmethods")
                        .stream().filter(single -> single.getDynamicObject("tpv_lc").getString("billno")
                                .equals(billno))
                        .map(single -> single.getBigDecimal("tpv_money"))
                        .reduce(BigDecimal.ZERO, BigDecimal::add));
        totalAmount.ifPresent(amount -> reasonable = amount.compareTo(end) > 0);
    }

    /**
     * @author: Heisenberg
     * @date: 2023/11/25 15:50
     * @description: 更新汇率，金额
     */
    private void calculatRate() {
        DynamicObject usd = (DynamicObject) this.getModel().getValue(USD);
        DynamicObject rmb = (DynamicObject) this.getModel().getValue(RMB);
        DynamicObject currency = (DynamicObject) this.getModel().getValue(CURRENCY);
        Date rateDate = (Date) this.getModel().getValue(CURRENT_DATE);
        DynamicObject rateTable = (DynamicObject) this.getModel().getValue(RATE_TABLE);

        // 开证折RMB汇率
        BigDecimal exchangeRateCny = YEM.getExchangeRate(rateTable, currency, rmb, rateDate);
        this.getModel().setValue(TO_RMBRATE, exchangeRateCny);
        // 开证转USD汇率
        BigDecimal exchangeRateUsd = YEM.getExchangeRate(rateTable, currency, usd, rateDate);
        this.getModel().setValue(TO_USDRATE, exchangeRateUsd);

        Optional.ofNullable(this.getModel().getValue(AMOUNT))
                .ifPresent(fee -> {
                    this.getModel().setValue(RMB_AMOUNT, exchangeRateCny.multiply((BigDecimal) fee));
                    this.getModel().setValue(USD_AMOUNT, exchangeRateUsd.multiply((BigDecimal) fee));
                });

    }
}
