package tpv.tpv_wm.tpv_iscb.java.api.sealstewardapi;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.itopview.wm.sys.utils.Api;
import com.itopview.wm.sys.utils.ApiHelper;
import com.itopview.wm.sys.utils.ApiLog;
import com.itopview.wm.utils.YEM;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;

/**
 * @Author: Heisenberg Wenbei He
 * @Project: zhck
 * @Package: com.itopview.wm.dt.api.sealstewardapi.SealStewardHelper
 * @Date: 10:39 2023/10/19
 * @Description: 章管家接口工具类
 * @Version: 1.0
 */
public final class SealStewardClientHelper {
    private static final String BILLSIGN = "tpv_generateprecode"; // 预审字符码单据标识
    private static final String ENTITY = "tpv_preapplycode"; // 预审字符码单据体标识
    private static final String PROCODE = "tpv_precode"; // 预审字符码中单据体字段预审字符码标识
    private static final String API = "api.seal"; // 章管家接口编码
    private static final Random RANDOM = new Random();

    private SealStewardClientHelper() {
        super();
    }

    /**
     * @return String
     * @author: Heisenberg
     * @date: 2023/10/19 11:28
     * @description: 【预审序列号】按 随机三位英文字母+发生时间的日期总秒数 传值，例：abc1234567891234
     */
    public static String generatePreApprovalNumber(Date date) {
        // 生成三位随机小写字母
        String randomLetters = generateRandomLetters();
        // 获取当前时间的日期总秒数
        long timestamp = date.getTime() / 1000;
        // 拼接预审序列号
        return randomLetters + timestamp;
    }

    private static String generateRandomLetters() {
        StringBuilder stringBuilder = new StringBuilder();

        for (int i = 0; i < 3; i++) {
            // 生成随机的小写字母ASCII码值，a的ASCII码值为97，z的ASCII码值为122
            int asciiValue = RANDOM.nextInt(26) + 97;
            char letter = (char) asciiValue;
            stringBuilder.append(letter);
        }

        return stringBuilder.toString();
    }

    /**
     * @return String
     * @author: Heisenberg
     * @date: 2023/10/23 9:09
     * @description: 生成一个随机的六位阿拉伯数字, 配合生成预审字符码
     */
    public static String generateUniqueSixDigitNumber() {
        // 生成一个随机的六位阿拉伯数字
        return String.format("%06d", RANDOM.nextInt(1000000));
    }

    /**
     * @param date: 当前业务日期
     * @return String
     * @author: Heisenberg
     * @date: 2023/10/23 9:03
     * @description: 随机六位阿拉伯数字， 需在一自然年内避免重复(预审字符码)
     */
    public static String generatePreApprovalCode(Date date) {
        // 获取当前年份
        SimpleDateFormat yearFormat = new SimpleDateFormat("yyyy");
        String currentYear = yearFormat.format(date);
        AtomicReference<String> preCode = new AtomicReference<>(generateUniqueSixDigitNumber());
        DynamicObject loaded = BusinessDataServiceHelper.loadSingle(YEM.getCurrentOrgId(),
                "bos_adminorg", "id,name"); // 获取当前登录人的所属部门
        DynamicObject preApprovalCode = BusinessDataServiceHelper.loadSingle(BILLSIGN,
                new QFilter[]{new QFilter("tpv_currentyear", QCP.equals, currentYear)});
        Optional.ofNullable(preApprovalCode)
                .map(u -> {
                    // preApprovalCode不为空时执行此处代码
                    boolean codeMatched = true;
                    while (codeMatched) {
                        codeMatched = u.getDynamicObjectCollection(ENTITY).stream()
                                .anyMatch(row -> preCode.get().equals(row.get(PROCODE)));
                        if (codeMatched) {
                            preCode.set(generateUniqueSixDigitNumber());
                        }
                    }
                    u.getDynamicObjectCollection(ENTITY).addNew().set(PROCODE, preCode.get());
                    SaveServiceHelper.save(new DynamicObject[]{preApprovalCode});
                    return u;
                })
                .orElseGet(() -> {
                    // preApprovalCode为空时执行此处代码
                    DynamicObject codeBill = BusinessDataServiceHelper.newDynamicObject(BILLSIGN);
                    codeBill.set("billno", "tpv" + currentYear);
                    codeBill.set("tpv_currentyear", currentYear);
                    codeBill.set("creator", YEM.getCurrentUserId()); // 赋值业务员为当前登陆人，用于带出当前部门
                    codeBill.set("tpv_operator", YEM.getCurrentUserId()); // 赋值创建人为当前登陆人
                    codeBill.set("org", loaded); // 赋值组织为当前部门
                    codeBill.set("billstatus", "C"); // 赋值单据状态为已审核
                    codeBill.getDynamicObjectCollection(ENTITY).addNew().set(PROCODE, preCode.get());
                    SaveServiceHelper.save(new DynamicObject[]{codeBill});
                    return codeBill;
                });
        return preCode.get();
    }

    /**
     * @param date:
     * @return String
     * @author: Heisenberg
     * @date: 2023/10/26 17:11
     * @description: 格式化Date
     */
    public static String dateFormat(Date date) {
        // 创建SimpleDateFormat对象，指定日期格式
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        return dateFormat.format(date);
    }

    public static String getLimitEndTime(Date date) {
        // 创建Calendar对象，并设置为当前时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);

        // 添加60天
        calendar.add(Calendar.DAY_OF_YEAR, 60);

        // 获取60天后的日期
        return dateFormat(calendar.getTime());
    }

    /**
     * @param apiKey: api编码
     * @return ApiLog 苍穹日志
     * @author: Heisenberg
     * @date: 2023/12/20 18:57
     * @description: 获取当前接口日志
     */
    public static Map<Object, Object> getApiLog(String apiKey) {
        ApiHelper apiHelper = new ApiHelper(API);
        ApiLog log = null;
        Map<Object, Object> map = new HashMap<>();
        if (apiHelper.isExist()) {
            Api api = apiHelper.getApi(apiKey);
            if (api.isExist()) {
                String url = api.getUrl();
                log = apiHelper.log(api);
                map.put("url", url);
                map.put("log", log);
            }
        }
        return map;
    }

    /**
     * @param url: 请求地址
     * @return String
     * @author: Heisenberg
     * @date: 2023/12/21 18:38
     * @description: get请求的域名放入到请求体中
     */
    public static String requestEntity(String url) {
        ObjectMapper objectMapper = new ObjectMapper();
        ObjectNode jsonNode = objectMapper.createObjectNode();
        jsonNode.put("url", url);
        return jsonNode.toString();
    }

    /**
     * @author: Heisenberg
     * @date: 2023/10/31 11:45
     * @description: 添加用印申请返回的结果（盖章结果、盖章码、用印编码）
     */
    public static class SealSteward {
        private String code;
        private String sealResultUrl;
        private String fast_seal_code;

        public String getCode() {
            return code;
        }

        public void setCode(String code) {
            this.code = code;
        }

        public String getSealResultUrl() {
            return sealResultUrl;
        }

        public void setSealResultUrl(String sealResultUrl) {
            this.sealResultUrl = sealResultUrl;
        }

        public String getFast_seal_code() {
            return fast_seal_code;
        }

        public void setFast_seal_code(String fast_seal_code) {
            this.fast_seal_code = fast_seal_code;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            if (o == null || getClass() != o.getClass()) {
                return false;
            }
            SealSteward that = (SealSteward) o;
            return Objects.equals(code, that.code)
                    && Objects.equals(sealResultUrl, that.sealResultUrl)
                    && Objects.equals(fast_seal_code, that.fast_seal_code);
        }

        @Override
        public int hashCode() {
            return Objects.hash(code, sealResultUrl, fast_seal_code);
        }

        @Override
        public String toString() {
            return "SealSteward{" +
                    "code='" + code + '\'' +
                    ", sealResultUrl='" + sealResultUrl + '\'' +
                    ", fast_seal_code='" + fast_seal_code + '\'' +
                    '}';
        }
    }

    /**
     * @author: Heisenberg
     * @date: 2023/10/31 11:46
     * @description: 批量添加修改用户
     */
    public static class UserForSeal {
        private String mobile;
        private String name;
        private String code;
        private String mail_address;
        private String password;
        private String position;
        private String uid;
        private String department_id;
        private int is_enable;
        private String cellphone;
        private String qyWechat_id;
        private String dingtalk_id;

        public String getMobile() {
            return mobile;
        }

        public void setMobile(String mobile) {
            this.mobile = mobile;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getCode() {
            return code;
        }

        public void setCode(String code) {
            this.code = code;
        }

        public String getMail_address() {
            return mail_address;
        }

        public void setMail_address(String mail_address) {
            this.mail_address = mail_address;
        }

        public String getPassword() {
            return password;
        }

        public void setPassword(String password) {
            this.password = password;
        }

        public String getPosition() {
            return position;
        }

        public void setPosition(String position) {
            this.position = position;
        }

        public String getUid() {
            return uid;
        }

        public void setUid(String uid) {
            this.uid = uid;
        }

        public String getDepartment_id() {
            return department_id;
        }

        public void setDepartment_id(String department_id) {
            this.department_id = department_id;
        }

        public int getIs_enable() {
            return is_enable;
        }

        public void setIs_enable(int is_enable) {
            this.is_enable = is_enable;
        }

        public String getCellphone() {
            return cellphone;
        }

        public void setCellphone(String cellphone) {
            this.cellphone = cellphone;
        }

        public String getQyWechat_id() {
            return qyWechat_id;
        }

        public void setQyWechat_id(String qyWechat_id) {
            this.qyWechat_id = qyWechat_id;
        }

        public String getDingtalk_id() {
            return dingtalk_id;
        }

        public void setDingtalk_id(String dingtalk_id) {
            this.dingtalk_id = dingtalk_id;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            UserForSeal that = (UserForSeal) o;
            return Objects.equals(mobile, that.mobile) && Objects.equals(name, that.name) && Objects.equals(code, that.code) && Objects.equals(mail_address, that.mail_address) && Objects.equals(password, that.password) && Objects.equals(position, that.position) && Objects.equals(uid, that.uid) && Objects.equals(department_id, that.department_id) && Objects.equals(is_enable, that.is_enable) && Objects.equals(cellphone, that.cellphone) && Objects.equals(qyWechat_id, that.qyWechat_id) && Objects.equals(dingtalk_id, that.dingtalk_id);
        }

        @Override
        public int hashCode() {
            return Objects.hash(mobile, name, code, mail_address, password, position, uid, department_id, is_enable, cellphone, qyWechat_id, dingtalk_id);
        }

        @Override
        public String toString() {
            return "UserForSeal{" +
                    "mobile='" + mobile + '\'' +
                    ", name='" + name + '\'' +
                    ", code='" + code + '\'' +
                    ", mail_address='" + mail_address + '\'' +
                    ", password='" + password + '\'' +
                    ", position='" + position + '\'' +
                    ", uid='" + uid + '\'' +
                    ", department_id='" + department_id + '\'' +
                    ", is_enable='" + is_enable + '\'' +
                    ", cellphone='" + cellphone + '\'' +
                    ", qyWechat_id='" + qyWechat_id + '\'' +
                    ", dingtalk_id='" + dingtalk_id + '\'' +
                    '}';
        }
    }

    /**
     * @author: Heisenberg
     * @date: 2023/10/31 11:47
     * @description: 批量添加修改部门
     */
    public static class DepartmentForSeal {
        private String name; // 部门名称 (required = true)
        private String code; // OA 系统部门编码
        private String parent_id; // OA 系统上级部门 id
        private String department_id; // OA 系统部门 id (required = true，repeatable = false ）
        private String remark; // 备注
        private int is_enable; // 部门状态

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getCode() {
            return code;
        }

        public void setCode(String code) {
            this.code = code;
        }

        public String getParent_id() {
            return parent_id;
        }

        public void setParent_id(String parent_id) {
            this.parent_id = parent_id;
        }

        public String getDepartment_id() {
            return department_id;
        }

        public void setDepartment_id(String department_id) {
            this.department_id = department_id;
        }

        public String getRemark() {
            return remark;
        }

        public void setRemark(String remark) {
            this.remark = remark;
        }

        public int getIs_enable() {
            return is_enable;
        }

        public void setIs_enable(int is_enable) {
            this.is_enable = is_enable;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            DepartmentForSeal that = (DepartmentForSeal) o;
            return is_enable == that.is_enable && Objects.equals(name, that.name) && Objects.equals(code, that.code) && Objects.equals(parent_id, that.parent_id) && Objects.equals(department_id, that.department_id) && Objects.equals(remark, that.remark);
        }

        @Override
        public int hashCode() {
            return Objects.hash(name, code, parent_id, department_id, remark, is_enable);
        }

        @Override
        public String toString() {
            return "DepartmentForSeal{" +
                    "name='" + name + '\'' +
                    ", code='" + code + '\'' +
                    ", parent_id='" + parent_id + '\'' +
                    ", department_id='" + department_id + '\'' +
                    ", remark='" + remark + '\'' +
                    ", is_enable=" + is_enable +
                    '}';
        }
    }
}
