package tpv.tpv_wm.tpv_iscb.java.api.shipnewsapi;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.itopview.wm.sys.utils.Api;
import com.itopview.wm.sys.utils.ApiHelper;
import com.itopview.wm.sys.utils.ApiLog;
import com.itopview.wm.utils.HttpHelper;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicReference;

import static java.util.Optional.ofNullable;

/**
 * @author: Heisenberg Wenbei He
 * @Project: zhkc
 * @Package: com.itopview.wm.dt.task.QueryFormShipNewsTask
 * @Date: 2023/09/06 16:25
 * @Version: 7.0
 * @Description: 调用船讯网接口进行 单船/多船查询操作
 */
public class ShipNewsClient {
    private static final int HTTP_OK = 0;
    private static String shipapikey;
    private static String rootUrl;
    private static String getSingleShip;
    private static String getManyShip;
    private static String getShipTrack;
    private static String queryShip;
    private static String max;

    static {
        ApiHelper apiHelper = new ApiHelper("api.shipxy"); // 获取苍穹注册的船讯网接口
        if (apiHelper.isExist()) {
            shipapikey = apiHelper.getParam("k"); // 获取船讯网接口密钥
            max = apiHelper.getParam("max"); // 允许查询返回的最大结果数

            Api api0 = apiHelper.getApi("url"); // 获取船讯网接口服务器地址
            Api api1 = apiHelper.getApi("GetManyShip"); // 多船查询
            Api api2 = apiHelper.getApi("GetShipTrack"); // 船舶轨迹查询
            Api api3 = apiHelper.getApi("QueryShip"); // 船舶搜索
            Api api4 = apiHelper.getApi("GetSingleShip"); // 单船查询

            if (api0.isExist()) {
                rootUrl = api0.getUrl();
            }
            if (api1.isExist()) {
                getManyShip = api1.getUrl();
            }
            if (api2.isExist()) {
                getShipTrack = api2.getUrl();
            }
            if (api3.isExist()) {
                queryShip = api3.getUrl();
            }
            if (api4.isExist()) {
                getSingleShip = api4.getUrl();
            }
        }
    }

    /**
     * @param shipIds: 船舶信息
     * @param params:  其他参数
     * @author: Heisenberg Wenbei He
     * @date: 2023/10/10 10:17
     * @description: 根据传参的数量判断执行不同的方法，拼接请求地址
     */
    public static Optional<List<ShipNewsClientHelper.ShipDetail>> queryShipInfo(List<String> shipIds, String... params) {
        AtomicReference<List<ShipNewsClientHelper.ShipDetail>> shipInfo = new AtomicReference<>();
        ofNullable(shipIds) // shipIds 为空返回null 不为空执行以下逻辑
                .filter(list -> !list.isEmpty())
                .ifPresent(ids -> { // ids 即 shipIds 不为空执行以下逻辑
                    int count = ids.size(); // 判断传入 ids 的大小决定调用方法
                    String idsString = String.join(",", ids); // 用 , 连接多个 ids
                    String shipUrlParams;
                    String method;
                    String url;
                    if (params.length == 1) {
                        method = "kw".equals(params[0]) ? queryShip : (count == 1 ? getSingleShip : getManyShip);
                        shipUrlParams = String.format("%s=%s&max=%s", params[0], idsString, max);
                    } else {
                        method = getShipTrack;
                        String beginTime = params[0];
                        String endTime = params[1];
                        shipUrlParams = String.format("cut=1&id=%s&btm=%s&etm=%s", idsString, beginTime, endTime);
                    }
                    url = String.format(rootUrl, method, shipapikey, shipUrlParams);
                    shipInfo.set(getShipInfo(url, method));
                });
        return Optional.ofNullable(shipInfo.get());
    }


    /**
     * @return List<ShipDetail> 船舶信息列表
     * @author: Heisenberg
     * @date: 2023/9/11 9:58
     * @description: 调用接口，返回的 json 通过反射封装到实体类 ShipDetail 中并返回
     */
    private static List<ShipNewsClientHelper.ShipDetail> getShipInfo(String url, String method) {
        Optional<ApiLog> optionalLog = getApiLog(method);
        List<ShipNewsClientHelper.ShipDetail> shipList = new ArrayList<>();
        // 发送HTTP GET请求并获取返回的json数据
        String response;
        int isSuccess = -1;
        try {
            response = HttpHelper.httpGet(url);
            // 解析json数据
            JSONObject jsonObject = JSONObject.parseObject(response);
            // 获取状态码
            isSuccess = Integer.parseInt(jsonObject.getString("status"));
            // 检查状态码是否为成功状态
            if (HTTP_OK != isSuccess) {
                int finalIsSuccess = isSuccess;
                optionalLog.ifPresent(log -> log.setErrorMsg(ShipNewsApiException.getErrMessage(finalIsSuccess)));
            }
            // 获取船舶详细信息 获取到data为船舶传讯 points为船舶轨迹查询
            JSONArray data = ofNullable(jsonObject.getJSONArray("data"))
                    .orElseGet(() -> jsonObject.getJSONArray("points"));
            // 反射生成ShipDetail对象并添加到shipList中
            data.stream().map(u -> JSONObject.parseObject(String.valueOf(u),
                    ShipNewsClientHelper.ShipDetail.class)).forEach(shipList::add);
            optionalLog.ifPresent(log -> {
                ObjectMapper objectMapper = new ObjectMapper();
                ObjectNode jsonNode = objectMapper.createObjectNode();
                jsonNode.put("url", url);
                log.setRequestData(jsonNode.toString());
                log.setResponseData(response);
            });
        } catch (Exception e) {
            int finalIsSuccess = isSuccess;
            optionalLog.ifPresent(log -> log.setErrorMsg(ShipNewsApiException.getErrMessage(finalIsSuccess)));
        }
        int finalIsSuccess = isSuccess;
        optionalLog.ifPresent(log -> {
            log.setSuccess(HTTP_OK == finalIsSuccess);
            log.write();
        });
        return shipList;
    }

    /**
     * @param apiKey: api编码
     * @return ApiLog 苍穹日志
     * @author: Heisenberg
     * @date: 2023/12/20 18:57
     * @description: 获取当前接口日志
     */
    private static Optional<ApiLog> getApiLog(String apiKey) {
        ApiHelper apiHelper = new ApiHelper("api.shipxy");
        ApiLog log = null;
        if (apiHelper.isExist()) {
            Api api = apiHelper.getApi(apiKey);
            if (api.isExist()) {
                log = apiHelper.log(api);
            }
        }
        return Optional.ofNullable(log);
    }
}
